import 'dart:convert';

import 'album_more_info.dart';

class Data {
  final String id;
  final String title;
  final String subtitle;
  final String type;
  final String image;
  final String perma_url;
  final Album_More_info more_info;
  final String explicit_content;
  final bool mini_obj;
  final String description;
  Data({
    required this.id,
    required this.title,
    required this.subtitle,
    required this.type,
    required this.image,
    required this.perma_url,
    required this.more_info,
    required this.explicit_content,
    required this.mini_obj,
    required this.description,
  });

  Data copyWith({
    String? id,
    String? title,
    String? subtitle,
    String? type,
    String? image,
    String? perma_url,
    Album_More_info? more_info,
    String? explicit_content,
    bool? mini_obj,
    String? description,
  }) {
    return Data(
      id: id ?? this.id,
      title: title ?? this.title,
      subtitle: subtitle ?? this.subtitle,
      type: type ?? this.type,
      image: image ?? this.image,
      perma_url: perma_url ?? this.perma_url,
      more_info: more_info ?? this.more_info,
      explicit_content: explicit_content ?? this.explicit_content,
      mini_obj: mini_obj ?? this.mini_obj,
      description: description ?? this.description,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'title': title,
      'subtitle': subtitle,
      'type': type,
      'image': image,
      'perma_url': perma_url,
      'more_info': more_info.toMap(),
      'explicit_content': explicit_content,
      'mini_obj': mini_obj,
      'description': description,
    };
  }

  factory Data.fromMap(Map<String, dynamic> map) {
    return Data(
      id: map['id'],
      title: map['title'],
      subtitle: map['subtitle'],
      type: map['type'],
      image: map['image'],
      perma_url: map['perma_url'],
      more_info: Album_More_info.fromMap(map['more_info']),
      explicit_content: map['explicit_content'],
      mini_obj: map['mini_obj'],
      description: map['description'],
    );
  }

  String toJson() => json.encode(toMap());

  factory Data.fromJson(String source) => Data.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Data(id: $id, title: $title, subtitle: $subtitle, type: $type, image: $image, perma_url: $perma_url, more_info: $more_info, explicit_content: $explicit_content, mini_obj: $mini_obj, description: $description)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is Data &&
        other.id == id &&
        other.title == title &&
        other.subtitle == subtitle &&
        other.type == type &&
        other.image == image &&
        other.perma_url == perma_url &&
        other.more_info == more_info &&
        other.explicit_content == explicit_content &&
        other.mini_obj == mini_obj &&
        other.description == description;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        title.hashCode ^
        subtitle.hashCode ^
        type.hashCode ^
        image.hashCode ^
        perma_url.hashCode ^
        more_info.hashCode ^
        explicit_content.hashCode ^
        mini_obj.hashCode ^
        description.hashCode;
  }
}
