import 'rights.dart';

import 'artist_map.dart';

class MoreInfo {
  String? music;
  String? albumId;
  String? album;
  String? label;
  String? origin;
  bool? isDolbyContent;
  String? s320kbps;
  String? encryptedMediaUrl;
  String? encryptedCacheUrl;
  String? albumUrl;
  String? duration;
  Rights? rights;
  String? cacheState;
  String? hasLyrics;
  String? lyricsSnippet;
  String? starred;
  String? copyrightText;
  ArtistMap? artistMap;
  String? releaseDate;
  String? labelUrl;
  bool? trillerAvailable;
  String? lyricsId;
  String? vcode;
  String? vlink;
  MoreInfo({
    this.music,
    this.albumId,
    this.album,
    this.label,
    this.origin,
    this.isDolbyContent,
    this.s320kbps,
    this.encryptedMediaUrl,
    this.encryptedCacheUrl,
    this.albumUrl,
    this.duration,
    this.rights,
    this.cacheState,
    this.hasLyrics,
    this.lyricsSnippet,
    this.starred,
    this.copyrightText,
    this.artistMap,
    this.releaseDate,
    this.labelUrl,
    this.trillerAvailable,
    this.lyricsId,
    this.vcode,
    this.vlink,
  });

  MoreInfo.fromJson(Map<String?, dynamic> json) {
    music = json['music'];
    albumId = json['album_id'];
    album = json['album'];
    label = json['label'];
    origin = json['origin'];
    isDolbyContent = json['is_dolby_content'];
    s320kbps = json['320kbps'];
    encryptedMediaUrl = json['encrypted_media_url'];
    encryptedCacheUrl = json['encrypted_cache_url'];
    albumUrl = json['album_url'];
    duration = json['duration'];
    rights =
        json['rights'] != null ? new Rights.fromJson(json['rights']) : null;
    cacheState = json['cache_state'];
    hasLyrics = json['has_lyrics'];
    lyricsSnippet = json['lyrics_snippet'];
    starred = json['starred'];
    copyrightText = json['copyright_text'];
    artistMap = json['artistMap'] != null
        ? new ArtistMap.fromJson(json['artistMap'])
        : null;
    releaseDate = json['release_date'];
    labelUrl = json['label_url'];
    trillerAvailable = json['triller_available'];
    lyricsId = json['lyrics_id'];
    vcode = json['vcode'];
    vlink = json['vlink'];
  }

  Map<String?, dynamic> toJson() {
    final Map<String?, dynamic> data = new Map<String?, dynamic>();
    data['music'] = this.music;
    data['album_id'] = this.albumId;
    data['album'] = this.album;
    data['label'] = this.label;
    data['origin'] = this.origin;
    data['is_dolby_content'] = this.isDolbyContent;
    data['320kbps'] = this.s320kbps;
    data['encrypted_media_url'] = this.encryptedMediaUrl;
    data['encrypted_cache_url'] = this.encryptedCacheUrl;
    data['album_url'] = this.albumUrl;
    data['duration'] = this.duration;
    if (this.rights != null) {
      data['rights'] = this.rights!.toJson();
    }
    data['cache_state'] = this.cacheState;
    data['has_lyrics'] = this.hasLyrics;
    data['lyrics_snippet'] = this.lyricsSnippet;
    data['starred'] = this.starred;
    data['copyright_text'] = this.copyrightText;
    if (this.artistMap != null) {
      data['artistMap'] = this.artistMap!.toJson();
    }
    data['release_date'] = this.releaseDate;
    data['label_url'] = this.labelUrl;
    data['triller_available'] = this.trillerAvailable;
    data['lyrics_id'] = this.lyricsId;
    data['vcode'] = this.vcode;
    data['vlink'] = this.vlink;
    return data;
  }

  @override
  String toString() {
    return 'MoreInfo(music: $music, albumId: $albumId, album: $album, label: $label, origin: $origin, isDolbyContent: $isDolbyContent, s320kbps: $s320kbps, encryptedMediaUrl: $encryptedMediaUrl, encryptedCacheUrl: $encryptedCacheUrl, albumUrl: $albumUrl, duration: $duration, rights: $rights, cacheState: $cacheState, hasLyrics: $hasLyrics, lyricsSnippet: $lyricsSnippet, starred: $starred, copyrightText: $copyrightText, artistMap: $artistMap, releaseDate: $releaseDate, labelUrl: $labelUrl, trillerAvailable: $trillerAvailable, lyricsId: $lyricsId, vcode: $vcode, vlink: $vlink)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is MoreInfo &&
        other.music == music &&
        other.albumId == albumId &&
        other.album == album &&
        other.label == label &&
        other.origin == origin &&
        other.isDolbyContent == isDolbyContent &&
        other.s320kbps == s320kbps &&
        other.encryptedMediaUrl == encryptedMediaUrl &&
        other.encryptedCacheUrl == encryptedCacheUrl &&
        other.albumUrl == albumUrl &&
        other.duration == duration &&
        other.rights == rights &&
        other.cacheState == cacheState &&
        other.hasLyrics == hasLyrics &&
        other.lyricsSnippet == lyricsSnippet &&
        other.starred == starred &&
        other.copyrightText == copyrightText &&
        other.artistMap == artistMap &&
        other.releaseDate == releaseDate &&
        other.labelUrl == labelUrl &&
        other.trillerAvailable == trillerAvailable &&
        other.lyricsId == lyricsId &&
        other.vcode == vcode &&
        other.vlink == vlink;
  }

  @override
  int get hashCode {
    return music.hashCode ^
        albumId.hashCode ^
        album.hashCode ^
        label.hashCode ^
        origin.hashCode ^
        isDolbyContent.hashCode ^
        s320kbps.hashCode ^
        encryptedMediaUrl.hashCode ^
        encryptedCacheUrl.hashCode ^
        albumUrl.hashCode ^
        duration.hashCode ^
        rights.hashCode ^
        cacheState.hashCode ^
        hasLyrics.hashCode ^
        lyricsSnippet.hashCode ^
        starred.hashCode ^
        copyrightText.hashCode ^
        artistMap.hashCode ^
        releaseDate.hashCode ^
        labelUrl.hashCode ^
        trillerAvailable.hashCode ^
        lyricsId.hashCode ^
        vcode.hashCode ^
        vlink.hashCode;
  }

  MoreInfo copyWith({
    String? music,
    String? albumId,
    String? album,
    String? label,
    String? origin,
    bool? isDolbyContent,
    String? s320kbps,
    String? encryptedMediaUrl,
    String? encryptedCacheUrl,
    String? albumUrl,
    String? duration,
    Rights? rights,
    String? cacheState,
    String? hasLyrics,
    String? lyricsSnippet,
    String? starred,
    String? copyrightText,
    ArtistMap? artistMap,
    String? releaseDate,
    String? labelUrl,
    bool? trillerAvailable,
    String? lyricsId,
    String? vcode,
    String? vlink,
  }) {
    return MoreInfo(
      music: music ?? this.music,
      albumId: albumId ?? this.albumId,
      album: album ?? this.album,
      label: label ?? this.label,
      origin: origin ?? this.origin,
      isDolbyContent: isDolbyContent ?? this.isDolbyContent,
      s320kbps: s320kbps ?? this.s320kbps,
      encryptedMediaUrl: encryptedMediaUrl ?? this.encryptedMediaUrl,
      encryptedCacheUrl: encryptedCacheUrl ?? this.encryptedCacheUrl,
      albumUrl: albumUrl ?? this.albumUrl,
      duration: duration ?? this.duration,
      rights: rights ?? this.rights,
      cacheState: cacheState ?? this.cacheState,
      hasLyrics: hasLyrics ?? this.hasLyrics,
      lyricsSnippet: lyricsSnippet ?? this.lyricsSnippet,
      starred: starred ?? this.starred,
      copyrightText: copyrightText ?? this.copyrightText,
      artistMap: artistMap ?? this.artistMap,
      releaseDate: releaseDate ?? this.releaseDate,
      labelUrl: labelUrl ?? this.labelUrl,
      trillerAvailable: trillerAvailable ?? this.trillerAvailable,
      lyricsId: lyricsId ?? this.lyricsId,
      vcode: vcode ?? this.vcode,
      vlink: vlink ?? this.vlink,
    );
  }
}
