import 'dart:convert';

import 'package:flutter/foundation.dart';

import 'more_info.dart';
import 'song_list.dart';

class PlaylistInfo {
  final String id;
  final String title;
  final String subtitle;
  final String header_desc;
  final String type;
  final String perma_url;
  final String image;
  final String language;
  final String year;
  final String play_count;
  final String explicit_content;
  final String list_count;
  final String list_type;
  final List<SongList> list;
  final MoreInfo more_info;
  PlaylistInfo({
    required this.id,
    required this.title,
    required this.subtitle,
    required this.header_desc,
    required this.type,
    required this.perma_url,
    required this.image,
    required this.language,
    required this.year,
    required this.play_count,
    required this.explicit_content,
    required this.list_count,
    required this.list_type,
    required this.list,
    required this.more_info,
  });

  PlaylistInfo copyWith({
    String? id,
    String? title,
    String? subtitle,
    String? header_desc,
    String? type,
    String? perma_url,
    String? image,
    String? language,
    String? year,
    String? play_count,
    String? explicit_content,
    String? list_count,
    String? list_type,
    List<SongList>? list,
    MoreInfo? more_info,
  }) {
    return PlaylistInfo(
      id: id ?? this.id,
      title: title ?? this.title,
      subtitle: subtitle ?? this.subtitle,
      header_desc: header_desc ?? this.header_desc,
      type: type ?? this.type,
      perma_url: perma_url ?? this.perma_url,
      image: image ?? this.image,
      language: language ?? this.language,
      year: year ?? this.year,
      play_count: play_count ?? this.play_count,
      explicit_content: explicit_content ?? this.explicit_content,
      list_count: list_count ?? this.list_count,
      list_type: list_type ?? this.list_type,
      list: list ?? this.list,
      more_info: more_info ?? this.more_info,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'title': title,
      'subtitle': subtitle,
      'header_desc': header_desc,
      'type': type,
      'perma_url': perma_url,
      'image': image,
      'language': language,
      'year': year,
      'play_count': play_count,
      'explicit_content': explicit_content,
      'list_count': list_count,
      'list_type': list_type,
      'list': list,
      'more_info': more_info.toJson(),
    };
  }

  factory PlaylistInfo.fromMap(Map<String, dynamic> map) {
    return PlaylistInfo(
      id: map['id'],
      title: map['title'],
      subtitle: map['subtitle'],
      header_desc: map['header_desc'],
      type: map['type'],
      perma_url: map['perma_url'],
      image: map['image'],
      language: map['language'],
      year: map['year'],
      play_count: map['play_count'],
      explicit_content: map['explicit_content'],
      list_count: map['list_count'],
      list_type: map['list_type'],
      list: (map['list'] as List<dynamic>)
          .map((e) => SongList.fromJson(Map<String, dynamic>.from(e)))
          .toList(),
      more_info: MoreInfo.fromJson(map['more_info']),
    );
  }

  String toJson() => json.encode(toMap());

  factory PlaylistInfo.fromJson(String source) =>
      PlaylistInfo.fromMap(json.decode(source));

  @override
  String toString() {
    return 'PlaylistInfo(id: $id, title: $title, subtitle: $subtitle, header_desc: $header_desc, type: $type, perma_url: $perma_url, image: $image, language: $language, year: $year, play_count: $play_count, explicit_content: $explicit_content, list_count: $list_count, list_type: $list_type, list: $list, more_info: $more_info)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is PlaylistInfo &&
        other.id == id &&
        other.title == title &&
        other.subtitle == subtitle &&
        other.header_desc == header_desc &&
        other.type == type &&
        other.perma_url == perma_url &&
        other.image == image &&
        other.language == language &&
        other.year == year &&
        other.play_count == play_count &&
        other.explicit_content == explicit_content &&
        other.list_count == list_count &&
        other.list_type == list_type &&
        listEquals(other.list, list) &&
        other.more_info == more_info;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        title.hashCode ^
        subtitle.hashCode ^
        header_desc.hashCode ^
        type.hashCode ^
        perma_url.hashCode ^
        image.hashCode ^
        language.hashCode ^
        year.hashCode ^
        play_count.hashCode ^
        explicit_content.hashCode ^
        list_count.hashCode ^
        list_type.hashCode ^
        list.hashCode ^
        more_info.hashCode;
  }
}
