import 'dart:convert';

import 'shows.dart';
import 'top_query.dart';

import 'albums.dart';
import 'artists.dart';
import 'episodes.dart';
import 'playlist.dart';
import 'songs.dart';

class SearchResults {
  final Albums albums;
  final Songs songs;
  final Playlists playlists;
  final Artists artists;
  final Topquery topquery;
  final Shows shows;
  final Episodes episodes;
  SearchResults({
    required this.albums,
    required this.songs,
    required this.playlists,
    required this.artists,
    required this.topquery,
    required this.shows,
    required this.episodes,
  });

  SearchResults copyWith({
    Albums? albums,
    Songs? songs,
    Playlists? playlists,
    Artists? artists,
    Topquery? topquery,
    Shows? shows,
    Episodes? episodes,
  }) {
    return SearchResults(
      albums: albums ?? this.albums,
      songs: songs ?? this.songs,
      playlists: playlists ?? this.playlists,
      artists: artists ?? this.artists,
      topquery: topquery ?? this.topquery,
      shows: shows ?? this.shows,
      episodes: episodes ?? this.episodes,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'albums': albums.toMap(),
      'songs': songs.toMap(),
      'playlists': playlists.toMap(),
      'artists': artists.toMap(),
      'topquery': topquery.toMap(),
      'shows': shows.toMap(),
      'episodes': episodes.toMap(),
    };
  }

  factory SearchResults.fromMap(Map<String, dynamic> map) {
    return SearchResults(
      albums: Albums.fromMap(map['albums']),
      songs: Songs.fromMap(map['songs']),
      playlists: Playlists.fromMap(map['playlists']),
      artists: Artists.fromMap(map['artists']),
      topquery: Topquery.fromMap(map['topquery']),
      shows: Shows.fromMap(map['shows']),
      episodes: Episodes.fromMap(map['episodes']),
    );
  }

  String toJson() => json.encode(toMap());

  factory SearchResults.fromJson(String source) =>
      SearchResults.fromMap(json.decode(source));

  @override
  String toString() {
    return 'SearchResults(albums: $albums, songs: $songs, playlists: $playlists, artists: $artists, topquery: $topquery, shows: $shows, episodes: $episodes)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is SearchResults &&
        other.albums == albums &&
        other.songs == songs &&
        other.playlists == playlists &&
        other.artists == artists &&
        other.topquery == topquery &&
        other.shows == shows &&
        other.episodes == episodes;
  }

  @override
  int get hashCode {
    return albums.hashCode ^
        songs.hashCode ^
        playlists.hashCode ^
        artists.hashCode ^
        topquery.hashCode ^
        shows.hashCode ^
        episodes.hashCode;
  }
}

// ignore: camel_case_types
