import 'more_info.dart';

class SongList {
  String? id;
  String? title;
  String? subtitle;
  String? headerDesc;
  String? type;
  String? permaUrl;
  String? image;
  String? language;
  String? year;
  String? playCount;
  String? explicitContent;
  String? listCount;
  String? listType;
  String? list;
  MoreInfo? moreInfo;
  SongList({
    this.id,
    this.title,
    this.subtitle,
    this.headerDesc,
    this.type,
    this.permaUrl,
    this.image,
    this.language,
    this.year,
    this.playCount,
    this.explicitContent,
    this.listCount,
    this.listType,
    this.list,
    this.moreInfo,
  });

  SongList.fromJson(Map<String?, dynamic> json) {
    id = json['id'];
    title = json['title'];
    subtitle = json['subtitle'];
    headerDesc = json['header_desc'];
    type = json['type'];
    permaUrl = json['perma_url'];
    image = json['image'];
    language = json['language'];
    year = json['year'];
    playCount = json['play_count'];
    explicitContent = json['explicit_content'];
    listCount = json['list_count'];
    listType = json['list_type'];
    list = json['list'];
    moreInfo = (json['more_info'] != null
        ? new MoreInfo.fromJson(json['more_info'])
        : null)!;
  }

  Map<String?, dynamic> toJson() {
    final Map<String?, dynamic> data = new Map<String?, dynamic>();
    data['id'] = this.id;
    data['title'] = this.title;
    data['subtitle'] = this.subtitle;
    data['header_desc'] = this.headerDesc;
    data['type'] = this.type;
    data['perma_url'] = this.permaUrl;
    data['image'] = this.image;
    data['language'] = this.language;
    data['year'] = this.year;
    data['play_count'] = this.playCount;
    data['explicit_content'] = this.explicitContent;
    data['list_count'] = this.listCount;
    data['list_type'] = this.listType;
    data['list'] = this.list;
    if (this.moreInfo != null) {
      data['more_info'] = this.moreInfo!.toJson();
    }
    return data;
  }

  SongList copyWith({
    String? id,
    String? title,
    String? subtitle,
    String? headerDesc,
    String? type,
    String? permaUrl,
    String? image,
    String? language,
    String? year,
    String? playCount,
    String? explicitContent,
    String? listCount,
    String? listType,
    String? list,
    MoreInfo? moreInfo,
  }) {
    return SongList(
      id: id ?? this.id,
      title: title ?? this.title,
      subtitle: subtitle ?? this.subtitle,
      headerDesc: headerDesc ?? this.headerDesc,
      type: type ?? this.type,
      permaUrl: permaUrl ?? this.permaUrl,
      image: image ?? this.image,
      language: language ?? this.language,
      year: year ?? this.year,
      playCount: playCount ?? this.playCount,
      explicitContent: explicitContent ?? this.explicitContent,
      listCount: listCount ?? this.listCount,
      listType: listType ?? this.listType,
      list: list ?? this.list,
      moreInfo: moreInfo ?? this.moreInfo,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is SongList &&
        other.id == id &&
        other.title == title &&
        other.subtitle == subtitle &&
        other.headerDesc == headerDesc &&
        other.type == type &&
        other.permaUrl == permaUrl &&
        other.image == image &&
        other.language == language &&
        other.year == year &&
        other.playCount == playCount &&
        other.explicitContent == explicitContent &&
        other.listCount == listCount &&
        other.listType == listType &&
        other.list == list &&
        other.moreInfo == moreInfo;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        title.hashCode ^
        subtitle.hashCode ^
        headerDesc.hashCode ^
        type.hashCode ^
        permaUrl.hashCode ^
        image.hashCode ^
        language.hashCode ^
        year.hashCode ^
        playCount.hashCode ^
        explicitContent.hashCode ^
        listCount.hashCode ^
        listType.hashCode ^
        list.hashCode ^
        moreInfo.hashCode;
  }

  @override
  String toString() {
    return 'SongList(id: $id, title: $title, subtitle: $subtitle, headerDesc: $headerDesc, type: $type, permaUrl: $permaUrl, image: $image, language: $language, year: $year, playCount: $playCount, explicitContent: $explicitContent, listCount: $listCount, listType: $listType, list: $list, moreInfo: $moreInfo)';
  }
}
