import 'dart:convert';

import 'package:flutter/foundation.dart';

import 'data.dart';

class Songs {
  final List<Data> data;
  final int position;
  Songs({
    required this.data,
    required this.position,
  });

  Songs copyWith({
    List<Data>? data,
    int? position,
  }) {
    return Songs(
      data: data ?? this.data,
      position: position ?? this.position,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'data': data.map((x) => x.toMap()).toList(),
      'position': position,
    };
  }

  factory Songs.fromMap(Map<String, dynamic> map) {
    return Songs(
      data: List<Data>.from(map['data']?.map((x) => Data.fromMap(x))),
      position: map['position']?.toInt(),
    );
  }

  String toJson() => json.encode(toMap());

  factory Songs.fromJson(String source) => Songs.fromMap(json.decode(source));

  @override
  String toString() => 'Songs(data: $data, position: $position)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is Songs &&
        listEquals(other.data, data) &&
        other.position == position;
  }

  @override
  int get hashCode => data.hashCode ^ position.hashCode;
}
