import 'package:flutter/foundation.dart';
import 'top_song.dart';

import 'modules.dart';
import 'more_info.dart';

class TopSongs {
  String? id;
  String? title;
  String? subtitle;
  String? headerDesc;
  String? type;
  String? permaUrl;
  String? image;
  String? language;
  String? year;
  String? playCount;
  String? explicitContent;
  String? listCount;
  String? listType;
  List<TopSong>? list;
  MoreInfo? moreInfo;
  Modules? modules;
  TopSongs({
    this.id,
    this.title,
    this.subtitle,
    this.headerDesc,
    this.type,
    this.permaUrl,
    this.image,
    this.language,
    this.year,
    this.playCount,
    this.explicitContent,
    this.listCount,
    this.listType,
    this.list,
    this.moreInfo,
    this.modules,
  });

  TopSongs.fromJson(Map<String?, dynamic> json) {
    id = json['id'];
    title = json['title'];
    subtitle = json['subtitle'];
    headerDesc = json['header_desc'];
    type = json['type'];
    permaUrl = json['perma_url'];
    image = json['image'];
    language = json['language'];
    year = json['year'];
    playCount = json['play_count'];
    explicitContent = json['explicit_content'];
    listCount = json['list_count'];
    listType = json['list_type'];
    if (json['list'] != null) {
      list = [];
      json['list'].forEach((v) {
        list!.add(new TopSong.fromJson(v));
      });
    }
    moreInfo = json['more_info'] != null
        ? new MoreInfo.fromJson(json['more_info'])
        : null;
  }

  Map<String?, dynamic> toJson() {
    final Map<String?, dynamic> data = new Map<String?, dynamic>();
    data['id'] = this.id;
    data['title'] = this.title;
    data['subtitle'] = this.subtitle;
    data['header_desc'] = this.headerDesc;
    data['type'] = this.type;
    data['perma_url'] = this.permaUrl;
    data['image'] = this.image;
    data['language'] = this.language;
    data['year'] = this.year;
    data['play_count'] = this.playCount;
    data['explicit_content'] = this.explicitContent;
    data['list_count'] = this.listCount;
    data['list_type'] = this.listType;
    if (this.list != null) {
      data['list'] = this.list!.map((v) => v.toJson()).toList();
    }
    if (this.moreInfo != null) {
      data['more_info'] = this.moreInfo!.toJson();
    }
    if (this.modules != null) {
      data['modules'] = this.modules!.toJson();
    }
    return data;
  }

  TopSongs copyWith({
    String? id,
    String? title,
    String? subtitle,
    String? headerDesc,
    String? type,
    String? permaUrl,
    String? image,
    String? language,
    String? year,
    String? playCount,
    String? explicitContent,
    String? listCount,
    String? listType,
    List<TopSong>? list,
    MoreInfo? moreInfo,
    Modules? modules,
  }) {
    return TopSongs(
      id: id ?? this.id,
      title: title ?? this.title,
      subtitle: subtitle ?? this.subtitle,
      headerDesc: headerDesc ?? this.headerDesc,
      type: type ?? this.type,
      permaUrl: permaUrl ?? this.permaUrl,
      image: image ?? this.image,
      language: language ?? this.language,
      year: year ?? this.year,
      playCount: playCount ?? this.playCount,
      explicitContent: explicitContent ?? this.explicitContent,
      listCount: listCount ?? this.listCount,
      listType: listType ?? this.listType,
      list: list ?? this.list,
      moreInfo: moreInfo ?? this.moreInfo,
      modules: modules ?? this.modules,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is TopSongs &&
        other.id == id &&
        other.title == title &&
        other.subtitle == subtitle &&
        other.headerDesc == headerDesc &&
        other.type == type &&
        other.permaUrl == permaUrl &&
        other.image == image &&
        other.language == language &&
        other.year == year &&
        other.playCount == playCount &&
        other.explicitContent == explicitContent &&
        other.listCount == listCount &&
        other.listType == listType &&
        listEquals(other.list, list) &&
        other.moreInfo == moreInfo &&
        other.modules == modules;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        title.hashCode ^
        subtitle.hashCode ^
        headerDesc.hashCode ^
        type.hashCode ^
        permaUrl.hashCode ^
        image.hashCode ^
        language.hashCode ^
        year.hashCode ^
        playCount.hashCode ^
        explicitContent.hashCode ^
        listCount.hashCode ^
        listType.hashCode ^
        list.hashCode ^
        moreInfo.hashCode ^
        modules.hashCode;
  }
}
